from rest_framework import generics
from rest_framework.fields import SerializerMethodField
from rest_framework.permissions import AllowAny
from rest_framework.serializers import ModelSerializer
from wagtail_rest_pack.exception.handler import custom_exception_handler

from wagtail_rest_pack.comments.content_type import create_content_type_id
from wagtail_rest_pack.comments.models import Comment


class GetCommentSerializer(ModelSerializer):
    children = SerializerMethodField('_get_children')

    class Meta:
        model = Comment
        fields = ['id', 'is_staff', 'name', 'parent', 'created_on', 'created_by', 'updated_on', 'updated_by', 'body',
                  'children']

    def _get_children(self, obj):
        children = Comment.objects.filter(parent_id=obj.id).order_by('created_on')
        serializer = GetCommentSerializer(children, many=True)
        return serializer.data


class ListCommentAPIView(generics.ListAPIView):
    """
    example: /?content_type=wagtailcore.Page&object_id=4
    """
    permission_classes = [AllowAny]
    serializer_class = GetCommentSerializer

    def get_queryset(self):
        object_id = self.request.query_params.get('object_id', None)
        content_type = self.request.query_params.get('content_type', None)
        content_type_id = create_content_type_id(object_id, content_type)
        return Comment.objects.filter(object_id=object_id, content_type_id=content_type_id, parent=None)

    def get_exception_handler(self):
        return custom_exception_handler



from rest_framework.exceptions import ValidationError
from django.core.exceptions import ValidationError as CoreDjangoValidationError
from rest_framework.response import Response
from rest_framework.views import exception_handler
from wagtail.api.v2.utils import BadRequestError
from rest_framework import status

def custom_exception_handler(exc, context):
    if isinstance(exc, (BadRequestError, ValidationError, CoreDjangoValidationError)):
        data = {'message': str(exc)}
        return Response(data, status=status.HTTP_400_BAD_REQUEST)
    response = exception_handler(exc, context)
    # Now add the HTTP status code to the response.
    if response is not None:
        response.data['status_code'] = response.status_code
    return response


from abc import ABC

from rest_framework.fields import Field

from django.conf import settings
from wagtail.images.api.fields import ImageRenditionField


def create_banner(page):
    banner = {}
    if hasattr(page, 'banner_title'):
        banner['title'] = page.banner_title
    if hasattr(page, 'banner_subtitle'):
        banner['subtitle'] = page.banner_subtitle
    if hasattr(page, 'banner_image'):
        spec = getattr(settings, 'IMAGE_BANNER_RENDERITION', 'fill-300x200')
        banner['image'] = ImageRenditionField(spec).to_representation(page.banner_image)
    return banner


class BanneredChildrenSerializer(Field, ABC):

    def to_representation(self, value):
        request = self.context['request']
        qs = value.specific()
        order = request.query_params.get('order', None)
        if order is not None:
            qs = qs.order_by(order)
        qs = self.context['view'].paginate_queryset(qs)
        for page in qs:
            yield {
                'id': page.id,
                'slug': page.slug,
                'url': page.url,
                'last_published_at': page.last_published_at,
                'banner': create_banner(page)
            }



# Generated by Django 3.1.8 on 2021-04-30 06:37

from django.db import migrations, models
import django.db.models.deletion
import modelcluster.fields
import wagtail.core.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0060_fix_workflow_unique_constraint'),
        ('wagtailimages', '0023_add_choose_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='ImageSliderItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', wagtail.core.fields.RichTextField(default='', help_text='Text zobrazený přes obrázek.', max_length=200)),
                ('image', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.PROTECT, to='wagtailimages.image')),
            ],
            options={
                'verbose_name': 'Carousel Item',
                'verbose_name_plural': 'Carousel Items',
            },
        ),
        migrations.CreateModel(
            name='SiteSettingsSection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('page', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='carouselitems', to='wagtailcore.page')),
                ('slider', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='sitesettings.imageslideritem')),
            ],
            options={
                'verbose_name': 'Carousel Item',
                'verbose_name_plural': 'Carousel Items',
            },
        ),
    ]



from django.conf import settings
from rest_framework.exceptions import ValidationError
from importlib import import_module

def isRequestSeamlessWalk(request):
    from django.conf import settings
    return get_client_ip(request) in getattr(settings, "SEAMLESS_WALK_IPS", [])

def get_client_ip(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip

class RecaptchaVerifier:

    def verify(self, request):
        pass


def get_recaptcha_instance() -> RecaptchaVerifier:
    class_str = getattr(settings, "RECAPTCHA_VERIFIER", None)
    if class_str is None:
        raise ValidationError("Please, specify RECAPTCHA_VERIFIER in settings.")

    try:
        module_path, class_name = class_str.rsplit('.', 1)
        module = import_module(module_path)
        return getattr(module, class_name)()
    except (ImportError, AttributeError) as e:
        raise ImportError(class_str)




from wagtail.api import APIField
from django.db import models
from wagtail.images.edit_handlers import ImageChooserPanel
from wagtail.images import get_image_model_string
from wagtail.admin.edit_handlers import FieldPanel, MultiFieldPanel
from wagtail.images.api.fields import ImageRenditionField
from django.conf import settings


class PageBannerMixin(models.Model):
    banner_title = models.TextField(max_length=100, blank=False, default="")
    banner_subtitle = models.TextField(max_length=100, blank=False, default="")
    banner_image = models.ForeignKey(get_image_model_string(), on_delete=models.PROTECT, blank=False, null=True,
                                     default=None)

    page_banner_panels = [
        MultiFieldPanel(
            heading="Page Banner",
            children=[
                FieldPanel('banner_title'),
                FieldPanel('banner_subtitle'),
                ImageChooserPanel('banner_image'),
            ]
        ),
    ]

    class Meta:
        abstract = True
